{**********************************************************}
{  wildfireZ Data Types and Enumeration Values             }
{                                                          }
{  wildfireZ version 2.0.1 - 2000.06.14                    }
{  Coded by Geoffrey Michaels                              }
{  email: GeoffreyM@UltratechCreations.com                 }
{                                                          }
{  (c) 1999-2000 Ultratech Creations                       }
{  wildfireZ is a trademark of Ultratech Creations.        }
{                                                          }
{  website: www.UltratechCreations.com                     }
{                                                          }
{  This library is free software; you can redistribute it  }
{  and/or modify it under the terms of the GNU Lesser      }
{  General Public License as published by the Free         }
{  Software Foundation; either version 2.1 of the License, }
{  or (at your option) any later version.                  }
{                                                          }
{  This library is distributed in the hope that it will    }
{  be useful, but WITHOUT ANY WARRANTY; without even the   }
{  implied warranty of MERCHANTABILITY or FITNESS FOR A    }
{  PARTICULAR PURPOSE.  See the GNU Lesser General Public  }
{  License for more details.                               }
{                                                          }
{  You should have received a copy of the GNU Lesser       }
{  General Public License along with this library; if not, }
{  write to the Free Software Foundation, Inc., 59 Temple  }
{  Place, Suite 330, Boston, MA  02111-1307  USA           }
{                                                          }
{**********************************************************}

unit WildfireZDataTypes;

interface

type TFunc = function(): integer; stdcall;

type TProc = procedure(); stdcall;

type TPlugin = record
  Name: string;
  Handle: LongWord;
  LoadModel: Integer;
  Load: TFunc;
  Unload: TFunc;
  Shutdown: TProc;
end;

type TPPlugin = ^TPlugin;

type THiresDateTime = record
  DateTime: Int64;
  Tick: Integer;// number of < seconds (232.8 picoseconds resolution)
end;

type TUnpackedHiresDateTime = record
  Year: Integer;
  Month: Integer;
  Day: Integer;
  IDLOffset: Integer;
  IDLMinutesOffset: Integer;
  DayOfWeek: Integer;
  Hour: Integer;
  Minute: Integer;
  Second: Integer;
  Tick: Integer;
end;

type TInteger128 = record
  Low: Int64;
  High: Int64;
end;

const

// core data types enumeration values
  zDataTypesMin = 0;
  zDataTypesMax = 32;
  // all are longints and equivalent to Pascal Integer
  zNullParam = 0;
  zInt8 = 1;
  zInt16 = 2;
  zInt32 = 3;
  zInt64 = 4;
  zInt128 = 5;
  zUint8 = 6;
  zUint16 = 7;
  zUint32 = 8;
  zUint64 = 9;
  zUint128 = 10;
  zBoo8 = 11;
  zBoo16 = 12;
  zBoo32 = 13;
  zBoo64 = 14;
  zBoo128 = 15;
  zPakboo8 = 16;
  zPakboo16 = 17;
  zPakboo32 = 18;
  zPakboo64 = 19;
  zPakboo128 = 20;
  zSingle = 21;
  zDouble = 22;
  zCurrency = 23;
  zDateTime = 24;
  zHiresDateTime = 25;
  zString = 26;
  zWString = 27;
  zPointer = 28;
  zWPointer = 29;
  zUnknown = 30;
  zError = 31;
  zFile = 32;

  // byte width for data types
  // USAGE: zSize(zDataType)
  // EXAMPLE:  tmpint := zSize[zInt32]
  //
  zSize: Array [0..32] of Integer = ( 0, 1, 2 , 4, 8, 16, 1, 2 , 4, 8,
    16, 1, 2 , 4, 8, 16, 1, 2 , 4, 8, 16, 4, 8, 8, 8, 12, 0, 0, 4, 8,
    0, 0 ,0);

// kernel environment state enumeration values
  esMinValue = 0;
  esMaxValue = 2;

  esDesign = 2;
  esOffline = 0;
  esRuntime = 1;

// game state enumeration values
  gsMinValue = 0;
  gsMaxValue = 7;

  gsOffline = 0;
  gsStartup = 1;
  gsIntro = 2;
  gsDemo = 3;
  gsPlaying = 4;
  gsPaused = 5;
  gsGameOver = 6;
  gsRuntime = 7;

// misc. kernel enum values
  WFZK_PluginNotFound = 0;
  WFZK_PluginOnAppPath = 1;
  WFZK_PluginLoaded = 2;
//  WFZK_PluginOnWFZPath = 3;

  WFZK_PluginLoadFailed = 0;
  WFZK_PluginLoadSuccessful = 1;
  WFZK_PluginUnloadFailed = 0;
  WFZK_PluginUnloadSuccessful = 1;

  WFZK_DynamicPlugin = 1;
  WFZK_ResidentPlugin = 0;

  // event runtime overloads
  WFZ_AssignEvent = 1;
  WFZ_TriggerEvent = 2;


  // error codes
  // this is a zero, not the letter O.
  eNoErrors = 0;

  // Kernel unit errors 
  eWFZK_CannotUnloadResidentPlugin = 1;
  eWFZK_DFCNotFound = 2;
  eWFZK_PluginNotAvailable = 3;
  eWFZK_PluginNotFound = 4;
  eWFZK_PluginNotLoaded = 5;
  eWFZK_PluginShutdownNotFound = 6;
  eWFZK_PluginLoadNotFound = 7;
  eWFZK_PluginUnloadNotFound = 8;
  eWFZK_IncorrectFileExtension = 9;
  eWFZK_KernelAlreadyRunning = 10;
  eWFZK_NoPluginsLoaded = 11;
  eWFZK_UnableToLoadPlugin = 12;

// Kernel Support unit errors
  eWFZK_YearLessThan1980 = 1;
  eWFZK_YearMoreThan4095 = 2;
  eWFZK_MonthLessThan01 = 3;
  eWFZK_MonthMoreThan12 = 4;
  eWFZK_DayLessThan01 = 5;
  eWFZK_DayMoreThan31 = 6;
  eWFZK_FebruaryMoreThan28Days = 7;
  eWFZK_FebruaryMoreThan29Days = 8;
  eWFZK_AprilMoreThan30Days = 9;
  eWFZK_JuneMoreThan30Days = 10;
  eWFZK_SeptemberMoreThan30Days = 11;
  eWFZK_NovemberMoreThan30Days = 12;
  eWFZK_IDLOffsetLessThan00 = 13;
  eWFZK_IDLOffsetMoreThan24 = 14;
  eWFZK_IDLMinutesOffsetLessThan00 = 15;
  eWFZK_IDLMinutesOffsetMoreThan59 = 16;
  eWFZK_DayOfWeekLessThan01 = 17;
  eWFZK_DayOfWeekMoreThan07 = 18;
  eWFZK_HourLessThan01 = 19;
  eWFZK_HourMoreThan23 = 20;
  eWFZK_MinuteLessThan01 = 21;
  eWFZK_MinuteMoreThan59 = 22;
  eWFZK_SecondLessThan01 = 23;
  eWFZK_SecondMoreThan59 = 24;


implementation

end.
