{**********************************************************}
{  WildfireZ Plugins Kernel DLL                            }
{                                                          }
{  wildfireZ version 2.0.1 - 2000.06.14                    }
{  Coded by Geoffrey Michaels                              }
{  email: GeoffreyM@UltratechCreations.com                 }
{                                                          }
{  (c) 1999-2000 Ultratech Creations                       }
{  wildfireZ is a trademark of Ultratech Creations.        }
{                                                          }
{  website: www.UltratechCreations.com                     }
{                                                          }
{  This library is free software; you can redistribute it  }
{  and/or modify it under the terms of the GNU Lesser      }
{  General Public License as published by the Free         }
{  Software Foundation; either version 2.1 of the License, }
{  or (at your option) any later version.                  }
{                                                          }
{  This library is distributed in the hope that it will    }
{  be useful, but WITHOUT ANY WARRANTY; without even the   }
{  implied warranty of MERCHANTABILITY or FITNESS FOR A    }
{  PARTICULAR PURPOSE.  See the GNU Lesser General Public  }
{  License for more details.                               }
{                                                          }
{  You should have received a copy of the GNU Lesser       }
{  General Public License along with this library; if not, }
{  write to the Free Software Foundation, Inc., 59 Temple  }
{  Place, Suite 330, Boston, MA  02111-1307  USA           }
{                                                          }
{**********************************************************}

unit WildfireZKernel;

interface

uses Classes, SysUtils, Windows, WildfireZSupport, WildfireZDataTypes;
  // exports
  procedure ClearWFZBuffer(); stdcall;
  function PluginAvailable(PluginName: PChar; var PluginHandle: Longword): integer; stdcall;
  function GetAppHandle(): LongWord; stdcall;
  function GetAppInstance(): Longint; stdcall;
  procedure GetAppPath(); stdcall;
  function GetDFC(PluginHandle: LongWord; DFC: PChar): Pointer; stdcall;
  function GetPluginCount(): Integer; stdcall;
  function GetEnvironmentState(): Integer; stdcall;
  function GetFrameRate():Integer; stdcall;
  function GetFramesRendered():Integer; stdcall;
  function GetGameState(): Integer; stdcall;
  function GetKernelVersion(): Integer; stdcall;
  procedure GetOSPath(); stdcall;
  procedure GetOSSystemPath(); stdcall;
  function GetWFZBuffer(): Pointer; stdcall;
  function GetWFZBufferContentsSize(): integer; stdcall;
  function GetWFZBufferSize(): integer; stdcall;
  function GetWFZError(): Integer; stdcall;
  procedure IncFramesRendered(); stdcall;
  function LoadPlugin(PluginName: PChar; LoadModel: Integer): LongWord; stdcall;
  procedure SetWFZBufferContentsSize(Size: integer); stdcall;
  procedure SetWFZBufferSize(Size: integer); stdcall;
  procedure SetWFZError(Error: Integer); stdcall;
  procedure SetGameState(GameState: Integer); stdcall;
  procedure ShutdownKernel(); stdcall;
  function StartupKernel(AppPath: PChar; AppInstance: Longint; AppHandle: LongWord;
    EnvironmentState: Integer): Pointer; stdcall;
  procedure UnloadPlugin(PluginName: PChar); stdcall;

var
  // objects
  PluginsList: TList;
  // global variables
  FAppHandle: LongWord;
  FAppInstance: Longint;
  FAppPath: string;
  FEnvironmentState: Integer;
  FError: Integer;
  FGameState: Integer;
  FKernelVersion: Integer = $02000000;
  FOSPath: string;
  FOSSystemPath: string;
  FPluginCount: Integer;
  FRenderEvents: Int64;
  FRenderEventPrevious: Int64;
  FRenderEventCurrent: Int64;
  FWFZBufferCOntentsSize: integer;
  tmpPlugin: TPPlugin;
  tmpPluginName: string;
  tmpint: integer;
  tmpString: string;
  WFZBuffer: TMemoryStream;

implementation

// exported routines

procedure ClearWFZBuffer(); stdcall;
begin
  if FEnvironmentState = esOffline then exit;
  if WFZBuffer.Size <> 8192 then WFZBuffer.SetSize(8192);
  ZeroMemory(WFZBuffer.Memory, WFZBuffer.Size);
end;

function PluginAvailable(PluginName: PChar; var PluginHandle: Longword): integer; stdcall;
var i: integer;
begin
  Result := WFZK_PluginNotFound;
  if FEnvironmentState = esOffline then exit;
  PluginHandle := 0;
  tmpPluginName := strpas(PluginName);
  if Length(tmpPluginName) = 0 then exit;
  tmpString := Lowercase(tmpPluginName);
  if Pos('.wfz', tmpString) = 0 then
    begin
      FError := eWFZK_IncorrectFileExtension;
      exit;
    end;
  tmpString := ExtractFileName(tmpPluginName);
  for i := 0 to FPluginCount-1 do
    begin
      tmpPlugin := PluginsList.Items[i];
      if tmpPlugin.Name = tmpPluginName then
        begin
          PluginHandle := tmpPlugin.Handle;
          Result := WFZK_PluginLoaded;
          exit;
        end;
    end;
  if FileExists(FAppPath + tmpPluginName) = True then
    begin
      Result := WFZK_PluginOnAppPath;
      exit;
    end;
{  if FileExists(FWFZPath + tmpPluginName) = True then
    begin
      Result := WFZK_PluginOnWFZPath;
      exit;
    end;}
end;

function GetAppHandle(): LongWord; stdcall;
begin
  Result := FAppHandle;
end;

function GetAppInstance: Longint; stdcall;
begin
  Result := FAppInstance;
end;

Procedure GetAppPath(); stdcall;
begin
  if FEnvironmentState = esOffline then exit;
  ZeroMemory(WFZBuffer.Memory, WFZBuffer.Size);
  StrPCopy(WFZBuffer.Memory, FAppPath);
  FWFZBufferContentsSize := Length(FAppPath) + 1; // include null char
end;

function GetDFC(PluginHandle: LongWord; DFC: PChar): Pointer; stdcall;
begin
  Result := nil;
  if FEnvironmentState = esOffline then exit;
  FError := eNoErrors;
  if FPluginCount < 1 then
    begin
      FError := eWFZK_NoPluginsLoaded;
      exit;
    end;
  Result := GetProcAddress(PluginHandle, DFC);
  if Result = nil then FError := eWFZK_DFCNotFound;
end;

function GetPluginCount(): Integer; stdcall;
begin
  Result := FPluginCount;
end;

function GetEnvironmentState(): Integer; stdcall;
begin
  Result := FEnvironmentState;
end;

function GetWFZError(): Integer; stdcall;
begin
  Result := FError;
end;

function GetFrameRate(): Integer; stdcall;
begin
  Result := 0;
  if FEnvironmentState = esOffline then exit;
  // if no RenderEvents have occured, then exit
  if (FRenderEventCurrent = 0) or
     (FRenderEventPrevious = 0) then exit;
  // divide time since last render event into elapsed time frequency
  // returns current frames per second
  Result := GetElapsedTimeFrequency div (FRenderEventCurrent -
    FRenderEventPrevious);
end;

function GetFramesRendered(): Integer; stdcall;
begin
  Result := FRenderEvents;
end;

function GetGameState(): Integer; stdcall;
begin
  Result := FGameState;
end;

function GetKernelVersion(): Integer; stdcall;
begin
  Result := FKernelVersion;
end;

procedure GetOSPath(); stdcall;
begin
  if FEnvironmentState = esOffline then exit;
  ZeroMemory(WFZBuffer.Memory, WFZBuffer.Size);
  StrPCopy(WFZBuffer.Memory, FOSPath);
  FWFZBufferContentsSize := Length(FOSPath) + 1; // include null char
end;

procedure GetOSSystemPath(); stdcall;
begin
  if FEnvironmentState = esOffline then exit;
  ZeroMemory(WFZBuffer.Memory, WFZBuffer.Size);
  StrPCopy(WFZBuffer.Memory, FOSSystemPath);
  FWFZBufferContentsSize := Length(FOSSystemPath) + 1; // include null char
end;

function GetWFZBuffer(): Pointer; stdcall;
begin
  Result := nil;
  if FEnvironmentState = esOffline then exit;
  Result := WFZBuffer.Memory;
end;

function GetWFZBufferContentsSize(): integer; stdcall;
begin
  Result := FWFZBufferContentsSize;
end;

function GetWFZBufferSize(): integer; stdcall;
begin
  Result := 0;
  if FEnvironmentState = esOffline then exit;
  Result := WFZBuffer.Size;
end;

procedure IncFramesRendered(); stdcall;
begin
  if FEnvironmentState = esOffline then exit;
  Inc(FRenderEvents); // total frames rendered
  FRenderEventPrevious := FRenderEventCurrent;
  FRenderEventCurrent := GetStartupElapsedTime;
end;

function LoadPlugin(PluginName: PChar; LoadModel: Integer): LongWord; stdcall;
var i: Integer;
PluginHandle: Longword;
begin
  Result := 0;
  if FEnvironmentState = esOffline then exit;
  FError := eNoErrors;
  tmpPluginName := string(PluginName);
  tmpString := LowerCase(tmpPluginName);
  if Pos('.wfz', tmpString) = 0 then
    begin
      FError := eWFZK_IncorrectFileExtension;
      exit;
    end;
  tmpint := PluginAvailable(PluginName, PluginHandle);
  if tmpint = WFZK_PluginNotFound then exit;
  // if loaded, return handle
  if tmpint = WFZK_PluginLoaded then
    begin
      Result := PluginHandle;
      exit;
    end;
  // not loaded, so load it
  tmpPlugin := nil;
  New(tmpPlugin);
  tmpPlugin^.Name := '';
  tmpPlugin^.Handle := 0;
  tmpPlugin^.LoadModel := 0;
  tmpPlugin^.Load := nil;
  tmpPlugin^.Unload := nil;
  tmpPlugin^.Shutdown := nil;
  if tmpint = WFZK_PluginOnAppPath then tmpString := FAppPath + tmpPluginName;
//  if tmpint = WFZK_PluginOnWFZPath then tmpString := FWFZPath + tmpPluginName;
  tmpPlugin^.Handle := LoadLibrary(PChar(tmpString));
  if tmpPlugin^.Handle = 0 then
    begin
      FError := eWFZK_UnableToLoadPlugin;
      Dispose(tmpPlugin);
      exit;
    end;
  @tmpPlugin^.Load := GetProcAddress(tmpPlugin^.Handle, 'Load');
  if @tmpPlugin^.Load = nil then
    begin
      FError := eWFZK_PluginLoadNotFound;
      FreeLibrary(tmpPlugin^.Handle);
      Dispose(tmpPlugin);
      exit;
    end;
  @tmpPlugin^.Unload := GetProcAddress(tmpPlugin^.Handle, 'Unload');
  if @tmpPlugin^.Unload = nil then
    begin
      FError := eWFZK_PluginUnloadNotFound;
      FreeLibrary(tmpPlugin^.Handle);
      Dispose(tmpPlugin);
      exit;
    end;
  @tmpPlugin^.Shutdown := GetProcAddress(tmpPlugin^.Handle, 'Shutdown');
  if @tmpPlugin^.Shutdown = nil then
    begin
      FError := eWFZK_PluginShutdownNotFound;
      FreeLibrary(tmpPlugin^.Handle);
      Dispose(tmpPlugin);
      exit;
    end;
  // all main funcs found, startup plugin
  if tmpPlugin^.Load <> WFZK_PluginLoadSuccessful then
    begin
      // FError already contains plugin's failure reason
      FreeLibrary(tmpPlugin^.Handle);
      Dispose(tmpPlugin);
      exit;
    end;
  // everything OK, add plugin to list
  tmpPlugin^.Name := ExtractFileName(tmpPluginName);
  if (LoadModel <> WFZK_ResidentPlugin) AND (LoadModel <> WFZK_DynamicPlugin) then
    LoadModel := WFZK_DynamicPlugin;
  tmpPlugin^.LoadModel := LoadModel;
  Inc(FPluginCount);
  Result := tmpPlugin^.Handle;
  PluginsList.Add(tmpPlugin);
end;

procedure SetWFZBufferContentsSize(Size: integer); stdcall;
begin
  if FEnvironmentState = esOffline then exit;
  FWFZBufferContentsSize := Size;
end;

procedure SetWFZBufferSize(Size: integer); stdcall;
begin
  if FEnvironmentState = esOffline then exit;
  if Size > 8192 then WFZBuffer.SetSize(Size);
end;

procedure SetWFZError(Error: Integer); stdcall;
begin
  if FEnvironmentState = esOffline then exit;
  FError := Error;
end;

procedure SetGameState(GameState: Integer); stdcall;
begin
  if FEnvironmentState = esOffline then exit;
  if (GameState < gsMinValue) OR (GameState > gsMaxValue) then exit;
  FGameState := GameState;
end;

procedure ShutdownKernel(); stdcall;
var I: Integer;
begin
  if FEnvironmentState = esOffline then exit;
  if FPluginCount <> 0 then
    begin
      for I := 0 to FPluginCount - 1 do
        begin
          tmpPlugin := PluginsList.Items[i];
          tmpPlugin^.Shutdown;
          FreeLibrary(tmpPlugin^.Handle);
          Dispose(tmpPlugin);
        end;
    end;
  PluginsList.Free;
  WFZBuffer.free;
  FAppHandle := 0;
  FAppInstance := 0;
  FAppPath := '';
  FEnvironmentState := esOffline;
  FError := eNoErrors;
  FGameState := gsOffline;
  FOSPath := '';
  FOSSystemPath := '';
  FPluginCount := 0;
  FRenderEvents := 0;
  FRenderEventPrevious := 0;
  FRenderEventCurrent := 0;
  FWFZBufferContentsSize := 0;
  tmpPluginName := '';
  tmpString := '';
  tmpint := 0;
end;

function StartupKernel(AppPath: PChar; AppInstance: Longint; AppHandle: LongWord;
  EnvironmentState: Integer): Pointer; stdcall;
var Buffer: array[0..MAX_PATH - 1] of Char;
  dummy: Boolean;
begin
  Result := nil;
  if FEnvironmentState <> esOffline then
    begin
      // kernel is already running, can't startup
      FError := eWFZK_KernelAlreadyRunning;
      exit;
    end;
  FError := eNoErrors;
  // init variables
  FAppHandle := AppHandle;
  FAppInstance := AppInstance;
  FAppPath := strpas(AppPath);
  if (EnvironmentState < esMinValue) OR (EnvironmentState > esMaxValue) then
    FEnvironmentState := esRuntime
  else
    FEnvironmentState := EnvironmentState;
  FError := eNoErrors;
  FGameState := gsOffline;
  FPluginCount := 0;
  SetString(FOSPath, Buffer, GetWindowsDirectory(Buffer, SizeOf(Buffer)));
  SetString(FOSSystemPath, Buffer, GetSystemDirectory(Buffer, SizeOf(Buffer)));
  FOSPath := FOSPath +'\';
  FOSSystemPath := FOSSystemPath +'\';
  if FAppPath[length(FAppPath)] <> '\' then FAppPath := FAppPath +'\';
  IsElapsedTimeAvailable; // init counter
  FRenderEvents := 0;
  FRenderEventPrevious := 0;
  FRenderEventCurrent := 0;
  // init objects
  PluginsList := TList.Create;
  WFZBuffer := TMemoryStream.Create;
  WFZBuffer.SetSize(8192);
  FWFZBufferContentsSize := 0;
  Result := @ShutdownKernel;
end;

procedure UnloadPlugin(PluginName: PChar); stdcall;
var i: Integer;
begin
  if FEnvironmentState = esOffline then exit;
  FError := eNoErrors;
  tmpPluginName := LowerCase(strpas(PluginName));
  if Pos('.wfz', tmpPluginName) = 0 then
    begin
      FError := eWFZK_IncorrectFileExtension;
      exit;
    end;
  if FPluginCount < 1 then
    begin
      FError := eWFZK_NoPluginsLoaded;
      exit;
    end;
  for i := 0 to FPluginCount - 1 do
    begin
      tmpPlugin := PluginsList.Items[i];
      if tmpPlugin^.Name = tmpPluginName then
        begin
          if (tmpPlugin^.LoadModel = WFZK_ResidentPlugin) then
            begin
              FError := eWFZK_CannotUnloadResidentPlugin;
              exit;
            end;
          if (tmpPlugin^.Unload <> WFZK_PluginUnloadSuccessful) then
            begin
              // Ferror already contains plugin;s reason for failure
              exit;
            end;
          FreeLibrary(tmpPlugin^.Handle);
          Dispose(tmpPlugin);
          PluginsList.Delete(i);
          Dec(FPluginCount);
          exit;
        end;
    end;
end;

end.
